/*
 * Microsemi Switchtec(tm) PCIe Management Library
 * Copyright (c) 2017, Microsemi Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 */

#ifndef LIBSWITCHTEC_MACROS_H
#define LIBSWITCHTEC_MACROS_H

#ifndef ARRAY_SIZE
#define ARRAY_SIZE(arr) (sizeof(arr) / sizeof((arr)[0]))
#endif

#define DIV_ROUND_UP(n, d) (((n) + (d) - 1) / (d))

struct range {
	int start, end, step;
};

#define _RANGE_CNT(start, end, step) DIV_ROUND_UP((end) - (start) + 1, (step))
#define RANGE_CNT(rng) _RANGE_CNT((rng)->start, (rng)->end, (rng)->step)
#define RANGE_TO_IDX(rng, x) DIV_ROUND_UP((x) - (rng)->start, (rng)->step)

#define for_range(i, rng) \
	for((i) = (rng)->start; (i) <= (rng)->end; (i) += (rng)->step)

#define for_rev_range(i, rng) \
	for((i) = (rng)->start + (RANGE_CNT(rng) - 1) * (rng)->step; \
	    (i) >= (rng)->start; (i) -= (rng)->step)

#endif
